import React from "react";
import { createContext, useEffect, useCallback } from "react";
import { useDeploymentById, useCommandById } from '../../../api/SwaggerApi';
import { filterDeprecatedNotifications } from "../../common/Helper";

export const notificationContext = createContext({ deploymentNotifications: [], watchDeployment: ()=>{}, clearAll: ()=>{} });
export const WATCHED_DEPLOYMENT_IDS = "WATCHED_DEPLOYMENT_IDS";
export const DEPLOYMENT_NOTIFICATIONS = "DEPLOYMENT_NOTIFICATIONS";
export const WATCHED_COMMAND_IDS = "WATCHED_COMMAND_IDS";
export const CURRENT_NOTIFICATION_VERSION = 1;
const DEPLOYMENT_STATUS_POLL_INTERVAL = 3000;
const COMMAND_STATUS_POLL_INTERVAL = 1000;
const TYPE_DEPLOYMENT = "deployment";
const TYPE_COMMAND = "command";

export function NotificationProvider({ children }) {
  const [deployment, getDeployment] = useDeploymentById();
  const [command, getCommand] = useCommandById();

  const watchDeployment = (watchedDeploymentId) => watchId(watchedDeploymentId, WATCHED_DEPLOYMENT_IDS);
  const watchCommand = (watchedCommandId) => watchId(watchedCommandId, WATCHED_COMMAND_IDS);

  const watchId = (watchedId, storageKey) => {
    let watchedIds = JSON.parse(localStorage.getItem(storageKey));
    if (watchedIds) {
      if(!watchedIds.includes(watchedId)) {
        localStorage.setItem(storageKey, JSON.stringify([watchedId].concat(watchedIds)));
      }
    } else {
      localStorage.setItem(storageKey, JSON.stringify([watchedId]));
    }
  }

  function removeId(id, storageKey) {
    let watchedIds = JSON.parse(localStorage.getItem(storageKey));
    if (watchedIds) {
      let idsAfterRemove = watchedIds.filter(function(item){ 
          return item !== id; 
      });
      if (idsAfterRemove.length === 0) {
        localStorage.removeItem(storageKey);
      } else {
        localStorage.setItem(storageKey, JSON.stringify(idsAfterRemove));
      }
    }
  }

  const clearAll = () => {
    localStorage.removeItem(DEPLOYMENT_NOTIFICATIONS);
  } 

  function deploymentNotifications() {
    let notifications = JSON.parse(localStorage.getItem(DEPLOYMENT_NOTIFICATIONS));
    if (!notifications) {
      return [];
    }
    return notifications;
  }

  function setDeploymentNotifications(notifications) {
    localStorage.setItem(DEPLOYMENT_NOTIFICATIONS, JSON.stringify(notifications));
  }
 
  function getStoredIds(storageKey) {
    let ids = JSON.parse(localStorage.getItem(storageKey));
    return ids ? ids.map(Number) : [];
  }

  function createNotification(type, id, status, message) {
    if (TYPE_DEPLOYMENT === type) {
      return {
        "version": CURRENT_NOTIFICATION_VERSION,
        "id": id,
        "type": type,
        "message": message,
        "status": status.toLowerCase(),
        "link": "/deployments/" + id
      };
    }

    return {
      "version": CURRENT_NOTIFICATION_VERSION,
      "id": id,
      "type": type,
      "message": message,
      "status": status.toLowerCase()
    };
  }

  const appendNotifications = useCallback((type, id, status, message) => {
    let notification = createNotification(type, id, status, message);
    let notifications = filterDeprecatedNotifications(deploymentNotifications());
    let relatedNotification = notifications.find(n => n.type === type && n.id === id);
    if (relatedNotification) {
      let index = notifications.indexOf(relatedNotification);
      if (index !== -1) {
        notifications[index] = notification;
      }
      setDeploymentNotifications(notifications);
    } else {
      setDeploymentNotifications([notification].concat(notifications));
    }
  }, [])

  useEffect(() => {
    function removeDeploymentId(id) {
      removeId(id, WATCHED_DEPLOYMENT_IDS);
    }

    const interval = setInterval(() => {
      const deploymentIds = getStoredIds(WATCHED_DEPLOYMENT_IDS);
      for (const deploymentId of deploymentIds) {
        getDeployment(deploymentId);
        if(deployment?.id === deploymentId && (deployment?.status === "QUEUED" || deployment?.status === "PENDING")) {
          appendNotifications(TYPE_DEPLOYMENT, deploymentId, deployment.status, 
            (deployment.undeployment ? "Undeployment" : "Deployment") + " for " + deployment.namingName + " has been queued");
        } else if(deployment?.id === deploymentId && deployment?.status === "RUNNING") {
          appendNotifications(TYPE_DEPLOYMENT, deploymentId, deployment.status, 
            (deployment.undeployment ? "Undeployment" : "Deployment") + " for " + deployment.namingName + " is currently being run");
        } else if(deployment?.id === deploymentId && deployment?.status === "FAILED") {
          appendNotifications(TYPE_DEPLOYMENT, deploymentId, deployment.status,
            (deployment.undeployment ? "Undeployment" : "Deployment") + " for " + deployment.namingName + " has failed");
          removeDeploymentId(deployment.id);
        } else if(deployment?.id === deploymentId && deployment?.status === "SUCCESSFUL") {
          appendNotifications(TYPE_DEPLOYMENT, deploymentId, deployment.status,
            (deployment.undeployment ? "Undeployment" : "Deployment") + " for " + deployment.namingName + " was successful");
          removeDeploymentId(deployment.id);
        }
      }
    }, DEPLOYMENT_STATUS_POLL_INTERVAL);
    return () => clearInterval(interval);
  }, [deployment, getDeployment, appendNotifications]);

  useEffect(() => {
    function removeCommandId(id) {
      removeId(id, WATCHED_COMMAND_IDS);
    }

    const interval = setInterval(() => {
      const commandIds = getStoredIds(WATCHED_COMMAND_IDS);
      for (const commandId of commandIds) {
        getCommand(commandId);
        if(command?.id === commandId && command?.status === "PENDING") {
          appendNotifications(TYPE_COMMAND, commandId, command.status, command.type + " command for " + command.iocName + " is pending");
        } else if(command?.id === commandId && command?.status === "RUNNING") {
          appendNotifications(TYPE_COMMAND, commandId, command.status, command.type + " command for " + command.iocName + " is currently being run");
        } else if(command?.id === commandId && command?.status === "FAILED") {
          appendNotifications(TYPE_COMMAND, commandId, command.status, command.type + " command for " + command.iocName + " has failed");
          removeCommandId(command.id);
        } else if(command?.id === commandId && command?.status === "SUCCESSFUL") {
          appendNotifications(TYPE_COMMAND, commandId, command.status, command.type + " command for " + command.iocName + " was successful");
          removeCommandId(command.id);
        }
      }
    }, COMMAND_STATUS_POLL_INTERVAL);
    return () => clearInterval(interval);
  }, [command, getCommand, appendNotifications]);

  return (
    <notificationContext.Provider value={{ deploymentNotifications, watchDeployment, watchCommand, clearAll }}>
      {children}
    </notificationContext.Provider>
  );
}