import React, { useContext, useState } from 'react';
import { makeStyles } from '@material-ui/core/styles';
import { Paper, Grid, FormControlLabel, Switch, Hidden, Typography } from '@material-ui/core';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { DeploymentsTable } from "../../components/deployments/DeploymentsTable";
import { DeploymentsList } from "../../components/deployments/DeploymentsList";
import { useDeploymentSearch, userContext } from '../../api/SwaggerApi';
import { initRequestParams } from '../../components/common/Helper';
import { useEffect } from 'react';
import { SearchBar } from '../../components/common/SearchBar/SearchBar';
import AccessControl from '../../components/auth/AccessControl';

const useStyles = makeStyles((theme) => ({
  root: {
    width: '100%'
  },
  paper: {
    marginBottom: theme.spacing(2),
  },
  formControl: {
    marginLeft: '5px',
    marginRight: '5px',
  }
}));

export function DeploymentsView() {
  const classes = useStyles();
  const [deployments, getDeployments, /*reset*/, loading] = useDeploymentSearch();
  const [query, setQuery] = useState("");
  const [selectedTab, setSelectedTab] = useState(0);
  const [ownOnly, setOwnOnly] = useState(false);
  const {user} = useContext(userContext);
  const [deploymentStatus, setDeploymentStatus] = useState(null);

  const handleTabChange = (event, tab) => {
    setSelectedTab(tab);

    if (tab === 0) {
      setDeploymentStatus(null);
    }
    else if (tab === 1) {
      setDeploymentStatus("RUNNING");
    }
    else if (tab === 2) {
      setDeploymentStatus("FINISHED");
    }
    else if (tab === 3) {
      setDeploymentStatus("QUEUED");
    }
  };

  const handleChangeOwn = (event) => {
    const own = event.target.checked;
    setOwnOnly(own);
  };

  const [lazyParams, setLazyParams] = useState({
    first: 0,
    rows: 10,
    page: 0
  });
  
  const [columnSort, setColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  const rowsPerPage = [5, 10, 20, 50];

  useEffect(() => {

    let requestParams = initRequestParams(lazyParams, query, columnSort);

    if(ownOnly) {
      requestParams.user = user?.loginName;
    }
  
    if(columnSort.sortField === "start") {
      requestParams.orderBy = "START_TIME";
    }
  
    if(columnSort.sortField === "user") {
      requestParams.orderBy = "CREATED_BY";
    }
  
    if(columnSort.sortField === "ioc") {
      requestParams.orderBy = "IOC_NAME";
    }
  
    if(columnSort.sortField === "version") {
      requestParams.orderBy = "GIT_REFERENCE";
    }
  
    if(deploymentStatus) {
      requestParams.status = deploymentStatus;
    }

    getDeployments(requestParams);

  }, [getDeployments, lazyParams, query, columnSort, user, deploymentStatus, ownOnly])

  const content = (
    <SearchBar search={setQuery} loading={loading}>
      <Hidden smUp>
        <DeploymentsList deployments={deployments.deploymentList} loading={loading}/>
      </Hidden>
      <Hidden xsDown>
        <DeploymentsTable
        deployments={deployments.deploymentList} loading={loading}
          totalCount={deployments.totalCount} lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort}
          rowsPerPage={rowsPerPage} />
      </Hidden>
    </SearchBar>
  );
  

  return (
    <Paper className={classes.paper}>
      <Grid container spacing={1} justify="flex-start">
        <Grid item xs={12} md={10}>
          <Tabs
            value={selectedTab}
            onChange={handleTabChange}
            indicatorColor="primary"
            textColor="primary">
            <Tab label={<Typography variant="h5">All</Typography>} />
            <Tab label={<Typography variant="h5">Running</Typography>} />
            <Tab label={<Typography variant="h5">Finished</Typography>} />
            <Tab label={<Typography variant="h5">Queued</Typography>} />
          </Tabs>
        </Grid>
        <Grid item xs={8} md={2}>
          <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
            renderNoAccess={() => <></>} >

            <FormControlLabel className={classes.formControl}
              control={<Switch checked={ownOnly} onChange={handleChangeOwn} />}
              label={<Typography variant="h5">My deployments</Typography>}
            />
          </AccessControl>
        </Grid>
        <Grid item xs={12} md={12}>
          {content}
        </Grid>
      </Grid>
    </Paper>
  );
}
